/* vi:cino=>4,g4,h4,+4,c1,C4,(4,u4,\:0,l1:shiftwidth=4:softtabstop=4:expandtab
 */
/*

Copyright (c) 2009, SFB/TR-55
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef GS_GURU_H_
#define GS_GURU_H_

#include <gs.h> /* lots of macros */
#include <sys/types.h> /* for pid_t */
#include <unistd.h>    /*  for pid_t */
#include <stdint.h>    /* for uint32_t */
#include <stdio.h>     /* for FILE */

/** Use with @a gsSendSignal to send a "true". */
#define GSTrue   GS_TRUE

/** Use with @a gsSendSignal to send a "false". */
#define GSFalse  GS_FALSE

#define GS_GURU_VERSION "libgsguru-4.3.0"

/** Get a new handle (file descriptor) for GS.
 * @return a negative value in case of failure,
 * a non-negative (the handle) value otherwise.
 */
int gsNewHandle();

/** Release a handle.
 * This routine configures the handle in a way, that
 * @c no @c kill signal will be set if the handle is released.
 * @param handle gs-handle.
 * @return 0 on success, -1 otherwise.
 */
int gsRelease(int handle);

/** Send a signal.
 * @param handle gs-handle.
 * @param signal
 *      Legitimate values for @a signal are @a GS_USER_XXX.
 * @return 0 on success, -1 otherwise.
 */
int gsSendSignal(int handle, unsigned char signal);

/** Receive a signal.
 * @param handle gs-handle.
 * @param signal Pointer to variable where signal will be stored.
 *  This will be one of @a GS_USER_XXX.
 * @return returns -1 in case of failure, 0 in case of success.
 *  If the handle is in non-blocking mode, -2 is retured if no
 *  data is availabe, and @a errno is set to @a EGAIN.
 */
int gsRecvSignal(int handle, unsigned char *signal);

/** Put the handle in blocking mode.
 * By default, a new handle will be already in blocking mode.
 * @param handle gs-handle.
 * @return 0 on success, -1 otherwise.
 */
int gsBlocking(int handle);

/** Put the handle in non-blocking mode.
 * @param handle gs-handle.
 * @return 0 on success, -1 otherwise.
 */
int gsNonBlocking(int handle);

/** Sign up for async events.
 * Enable asynchronous events for this handle.
 * This means, that the process gets a signal
 * (by default @c SIGIO) if any async. events
 * happen. Use @a gsSetAsyncEvents to choose which events.
 * @param handle gs-handle.
 * @return 0 on success, -1 otherwise.
 */
int gsAsyncEnroll(int handle);

/** Sign someone else up for async events.
 * Like @a gsAsyncEnroll, but use process with PID @a pid
 * instead of the pid returned by @a getpid().
 * @param handle gs-handle.
 * @param pid  PID of other process
 * @return 0 on success, -1 otherwise.
 */
int gsAsyncEnrollPid(int handle, pid_t pid);

/** Drop async events.
 * The handle won't deliver asynchronous events any more.
 * @param handle gs-handle.
 * @return 0 on success, -1 otherwise.
 */
int gsAsyncDrop(int handle);

/** Set Async Events.
 *
 * All bits of @a GS_STATUS_xxx can be used, but
 * in practise, only @a GS_STATUS_RX_KILL makes sense,
 * which is also the default.
 *
 * Note that this setting is stored on the device,
 * i.e., it affects all other processes on the same node,
 * holding a handle to the device.
 *
 * @param handle gs-handle.
 * @param events A bitwise OR of the above events.
 * @return 0 on success, -1 otherwise.
 */
int gsSetAsyncEvents(int handle, uint32_t events);

/** Get async events settings.
 * Get the settings stored on the device for
 * asynchronous readers.
 * @param handle gs-handle.
 * @param events The settings are stored here.
 * @result 0 on success, -1 otherwise.
 */
int gsGetAsyncEvents(int handle, uint32_t *events);

/** Set Sync Events.
 *
 * All bits of @a GS_STATUS_xxx can be used,
 * but only @a GS_STATUS_RX_VALID makes sense.
 *
 * Note that this setting is stored on the device,
 * i.e., it affects all other processes on the same node,
 * holding a handle to the device.
 *
 * @param handle gs-handle.
 * @param events A bitwise OR of the above events.
 * @result 0 on success, -1 otherwise.
 */
int gsSetSyncEvents(int handle, uint32_t events);

/** Get sync events settings.
 * Get the settings stored on the device for
 * synchronous (blocking) readers.
 * @param handle gs-handle.
 * @param events The settings are stored here.
 * @result 0 on success, -1 otherwise.
 */
int gsGetSyncEvents(int handle, uint32_t *events);

/** Reset GS logic in NWP.
 * This routine resets the GS logic in the controlbox
 * in the NWP. After this routine, the GS logic in the NWP
 * should be in the same state as in a newly loaded
 * image, except that hardware exceptions cannot be cleared.
 * @param handle gs-handle.
 * @return 0 on success, -1 otherwise.
 */
int gsHardReset(int handle);


/** Set some static variables in the driver.
 * These variables are static variables in the GS-driver.
 * Use this function only if you know exactly what you do.
 * However, these variables only keep track of some events,
 * and changing their value does not change the operation of the gs-device.
 * @param handle gs-handle.
 * @param overflow the overflow counter, i.e., number of signals
 *  not retrieved by the user.
 * @param kills number of kills received.
 * @param valids number of valids received.
 * @return 0 on success, -1 otherwise.
 */
int gsSetStatics(int handle, uint64_t overflow, uint64_t kills, uint64_t valids);

/** Set some static variables in the driver.
 * These variables are static variables in the GS-driver.
 * @param handle gs-handle.
 * @param overflow the overflow counter, i.e., number of signals
 *  not retrieved by the user.
 * @param kills number of kills received.
 * @param valids number of valids received.
 * @return 0 on success, -1 otherwise.
 */
int gsGetStatics(int handle, uint64_t *overflow, uint64_t *kills, uint64_t *valids);

/** Set the limit to the GS timeout counter in the FPGA.
 * This function tries to set the hard timeout, and does
 * some error checking. The maximum number the timeout limit
 * can be set to is given by
 * @a GS_CONFIG_TO_LIMIT and @a GS_CONFIG_TO_LIMIT_S.
 * See the file @a gs.h for details.
 * @param handle gs-handle.
 * @param ticks timeout-limit in ticks. See @a gs.h for details.
 * @return 0 on success, -1 otherwise.
 */
int gsSetHardTimeout(int handle, int ticks);

/** Get the limit to the GS timeout counter in the FPGA.
 * @param handle gs-handle.
 * @param ticks timeout-limit in ticks. See @a gs.h for details.
 * @return 0 on success, -1 otherwise.
 */
int gsGetHardTimeout(int handle, int *ticks);

/** Timed barrier with global reduction.
 * This routine performs a global redution, with acts as a barrier.
 * If all nodes send @a GSTrue, all nodes will receive @a GSTrue
 * after the barrier. If at least one node sends @a GSFalse, all nodes
 * will receive @a GSFalse. 
 *
 * This Routine blocks until either all nodes have reached the 
 * barrier or a @a GSKill has been received, or until the timeout
 * expired.
 *
 * This method works both in blocking or non-blocking mode.
 * In fact, the latency might be lower if non-blocking mode
 * is used.
 *
 * @param handle gs-handle
 * @param signal global signal - this is an in-out parameter,
 *  i.e., the value @a signal has before @a gsBarrier is 
 *  called should be either @a GSTrue or @a GSFalse.
 *  After the call to @a gsBarrier, @a signal is set to
 *  either @a GSTrue or @a GSFalse as a result of a global
 *  reduction.
 * @param timeout Either a timeout or @a NULL if the barrier
 *  should block indefinitely. The timing is implemented via
 *  @a select(), so check the manpage for @a select() if
 *  @a timeout is modified by the @a select() system call.<br />
 *  Usually, the type @a timeval looks something like this:<br />
 *  <pre>
 *      struct timeval {
 *          long tv_sec;
 *          long tv_usec;
 *      }
 *  </pre>
 *  where @a tv_sec represents seconds, and @a tv_usec represents microseconds.
 * @return 0 on success, -1 if a severe error occured,
 *  -2 if the timeout expired.
 */
int gsTimedBarrier(int handle, unsigned char *signal, struct timeval *timeout);

/** Timed broadcast of binary data.
 * This routine broadcasts @a length bytes of binary data through the
 * global signals. For a meaningful result, only one node should 
 * use @a gsTimedBCSend, all other nodes should use @a gsTimedBCRecv.
 *
 * In fact, @a data is an in-out parameter. If there are multiple
 * broadcast sender, then every node receives the result of a
 * bitwise AND operation of all senders' @a data.
 *
 * @param handle gs-handle
 * @param data binary data
 * @param length length of binary data in bytes
 * @param timeout timeout. See @a gsTimedBarrier for an explanation.
 * @return 0 in case of success, -1 if a severe error occured,
 *  -2 if the timeout expired.
 */
int gsTimedBCSend(int handle, unsigned char *data, int length, struct timeval *timeout);

/** Timed receive of a broadcast of binary data.
 * This routine is the counterpart of @a gsTimedBCSend.
 * @param handle gs-handle
 * @param data buffer for binary data
 * @param length length of binary data in bytes
 * @param timeout timeout. See @a gsTimedBarrier for an explanation.
 * @return 0 in case of success, -1 if a severe error occured,
 *  -2 if the timeout expired.
 */
int gsTimedBCRecv(int handle, unsigned char *data, int length, struct timeval *timeout);

/** Send a kill via softexception.
 * This routine sends a "kill" to all other nodes,
 * and sets the "softexc" bit.
 */
int gsKill(int handle);

/** Dump the proc device to stdout.
 * Just tries to write all the proc device to stdout.
 * This is not tied to a file descriptor, as there is
 * just one @a /proc/gs.
 * @param stream the stream where the output is written to.
 */
void gsDumpProcDevice(FILE *stream);

/** Dump the proc device to stdout.
 * Just like @a gsDumpProcDevice, but uses output routines from libqpace.
 * @param stream the stream where the output is written to.
 */
void gsDumpProcDeviceQpaceErr(void);

#endif /* defined GS_GURU_H_ */

